/*  integration_a.cpp
 * 	
 *	Using different integration methods
 *  to integrate f(x) = x^2. 
 *
 *  Part a) Integration over [0,1]
 *
 */

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cstdio>

using namespace std;

// Function to be integrated
double f(double x){
	return x*x ;
}

// Integration using trapezoidal rule
double integrate_trapez(int n){
	
	double h = 1./(n-1);
	
	double integral = 0.5*h*f(0.);
	for(int i = 1; i < n-1; i++){
		integral += h*f(i*h);
	}
	integral += 0.5*h*f(1.);
	
	return integral;
}

// Integration using Monte-Carlo method
double integrate_mc(int n){
	
	double integral = 0;
	double x;
	
	srand(time(0));
	
	for(int i = 1; i <= n; i++){
		x = rand()/(double)RAND_MAX;
		integral += f(x);
	}
	integral /= (double)n;
	
	return integral;
}

int main(){
	
	// Calculate and print integral values
	cout << setw(12) <<  "n" << setw(13) << "Trapezregel" << setw(13) << "Monte Carlo" << endl;

	for (int n = 2; n <= 1e8; n *= 2){
		printf("%12u %12.10f %12.10f\n", n, integrate_trapez(n), integrate_mc(n));
	}

	return 0;
}
