/*  integration_b.cpp
 * 	
 *	Using different integration methods
 *  to integrate f(x) = x^2. 
 *
 *  Part b) Integration over [a,b]
 *
 */

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cstdio>

using namespace std;

// Function to be integrated
double f(double x){
	return x*x;
}

double F(double a, double b){
	return (b*b*b - a*a*a)/3.0;
}

// Integration using trapezoidal rule over [a,b]
double integrate_trapez(int n, double a, double b){
	
	double h = (b-a)/(n-1);
	
	double integral = 0.5*h*f(a);
	for(int i = 1; i < n-1; i++){
		integral += h*f(a + i*h);
	}
	integral += 0.5*h*f(b);
	
	return integral;
}

// Integration using Monte-Carlo method over [a,b]
double integrate_mc(int n, double a, double b){
	
	double integral = 0;
	double x;
	
	srand(time(0));
  
    double factor = (b-a)/(double)RAND_MAX;
	
	for(int i = 1; i <= n; i++){
		x = a + rand() * factor;
		integral += f(x);
	}
	integral /= (double)n;
	
	return (b-a)*integral;
}

int main(int argc, char* argv[]){

    // Read input parameters from the command line
    if(argc != 3){
        cerr << "Usage: " << argv[0] << " a b " << endl;
        exit(1);
    }
  
    double a = atof(argv[1]);
    double b = atof(argv[2]);
  
    if( b <= a){
        cerr << "Error: b should be greater than a!" << endl;
        exit(1);
    }
	
    // Calculate and print integral values
    cout << setw(12) <<  "n" << setw(13) << "Trapezregel" << setw(13) << "Monte Carlo" << setw(13) << "Analytisch" << endl;

	for (int n = 2; n <= 1e8; n *= 2){
		printf("%12u %12.10f %12.10f %12.10f\n", n, integrate_trapez(n, a, b), integrate_mc(n, a, b), F(a,b));
	}

	return 0;
}
