/*  integration_c.cpp
 * 	
 *	Using different integration methods
 *  to integrate f(x) = x^2. 
 *
 *  Part c) Writing output to an external file.
 *
 */

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <fstream>

using namespace std;

// Function to be integrated
double f(double x){
	return x*x;
}

double F(double a, double b){
	return (b*b*b-a*a*a)/3.0;
}

// Integration using trapezoidal rule over [a,b]
double integrate_trapez(int n, double a, double b){
	
	double h = (b-a)/(n-1);
	
	double integral = 0.5*h*f(a);
	for(int i = 1; i < n-1; i++){
		integral += h*f(a + i*h);
	}
	integral += 0.5*h*f(b);
	
	return integral;
}

// Integration using Monte-Carlo method over [a,b]
double integrate_mc(int n, double a, double b){
	
	double integral = 0;
	double x;
	
	srand(time(0));
  
    double factor = (b-a)/(double)RAND_MAX;
	
	for(int i = 1; i <= n; i++){
		x = a + rand() * factor;
		integral += f(x);
	}
	integral /= (double)n;
	
	return (b-a)*integral;
}

int main(){

    // Read input parameters
    double a = 0;
    double b = -1;
    while (b < a) {
        cout << "Please specify the integration limits a and b (b > a):" << endl;
        cin >> a >> b;
    }

	
    // Calculate and print integral values to file
	ofstream filestream("integrals.txt");
	if(filestream.is_open()){
		filestream.precision(10);
		filestream << setw(12) <<  "n" << setw(15) << "Trapezregel" << setw(15) << "Monte Carlo" << endl;
		for (int n = 2; n <= 1e8; n *= 2){
			filestream << fixed << setw(12) << n << " " 
			           << setw(12) << integrate_trapez(n,a,b) << " " 
			           << setw(15) << integrate_mc(n,a,b) << setw(15) << F(a,b) << endl;
		}		
	}
	filestream.close();

	return 0;
}
