#include <iostream>
#include <random>
#include <cstdlib>
#include <functional> // for shorthand version

/*
 * Use the random header from the C++11 standard to generate random numbers
 * Compiliation: g++ -Wall -o randomnumbers_c++11 randomnumbers_c++11.cpp -std=c++0x
 */

using namespace std;

int main(){

	mt19937 rng(time(NULL));

	// Uniformly distributed over [0,1]
	uniform_real_distribution<double> uniform_dist(0.,1.);

	cout << "Uniform: " << endl;
	for(int i = 0; i < 10; ++i){
		cout << uniform_dist(rng) << endl;
	}
	
	// Normally distributed with mean 0, std deviation 1
	normal_distribution<double> normal_dist(0., 1.);

	cout << "Normal: " << endl;
	for(int i = 0; i < 10; ++i){
		cout << normal_dist(rng) << endl;
	}
	
	// Shorthand versions:
	auto uniform = bind(uniform_dist, rng);
	auto normal = bind(normal_dist, rng);
	cout << "Shorthand versions: " << endl;
	cout << uniform() << endl;
	cout << normal() << endl;
	
	return 0;
}
